// Craig's starter code for mini-putt.  It looks long, but there
// are a lot of comments.  And your job is to add code in just three
// places: values for speedNorm and friction, simulation code in the
// step() function, and putt initialization code in mouseReleased().

///////////////////////////////////////////////////////////
// Define some constants that control the game as a whole.

int holeRadius = 15;
int ballRadius = 10;

// In many games, the action occurs in distinct phases.  In
// this mini-putt game, you're either sitting idle in between
// strokes, or you're winding up for a stroke, or the ball is
// in motion, or you're celebrating the completion of a level
// before returning to the idle state.  Define some constants
// to help us distinguish which state we're in.  We'll never
// change these values -- they're "symbolic names" that we can
// use in the rest of the program.
int STATE_READY = 1;
int STATE_WINDUP = 2;
int STATE_SIMULATE = 3;
int STATE_CELEBRATE = 4;

// Now use a global variable to remember what the state is at
// this very moment.
int state;

// The smallest and greatest amount of power that a single shot can have.
// Used to make sure the user doesn't try to send the ball off into the
// stratosphere.
float minPower = 50;
float maxPower = 400;

//////////////////////////////////////////////////////////
// Global variables that are related to each particular
// level.

// The location of the hole
float holeX;
float holeY;

//////////////////////////////////////////////////////////
// Global variables related to the current state of the 
// physical simulation

// The current location of the ball.
float ballX;
float ballY;
// And the current velocity.
float velX;
float velY;

// Two constants that control the physical character of
// the simulation.  You'll want to fiddle with these numbers
// until you get a simulation you're happy with.
// TODO: choose reasonable values for these numbers.

// A scaling factor: how much initial velocity do you get
// for each pixel's worth of windup power?  Probably a smallish
// number between 0 and 1.
float speedNorm;
// Coefficient of friction (or damping, whichever you want).
// If you're doing something like damping as discussed in class,
// use similar numbers to the sample sketches.
float friction;

//////////////////////////////////////////////////////////
// And now, the code.

void setup()
{
  size(600, 500);
  noStroke();
  strokeWeight(2);
  
  // Get things going the first time by setting up a random
  // hole location.
  holeX = random( holeRadius, width - holeRadius );
  holeY = random( holeRadius, height - holeRadius );
  resetGame();
  
  state = STATE_READY;
}

// Given the current position of the ball, find a new
// position for the hole that's far away from it.
void resetGame()
{
  // Move the ball to the centre of the hole.  Then make
  // up a bunch of random hole locations and try to find
  // one that's far from the ball.
  ballX = holeX;
  ballY = holeY;
  
  for( int idx = 0; idx < 20; ++idx ) {
    holeX = random( holeRadius, width - holeRadius );
    holeY = random( holeRadius, height - holeRadius );
    if( dist( ballX, ballY, holeX, holeY ) > 200.0 ) {
      // Yay, found a far away hole!  Stop looping.
      break;
    }
  }
}

void drawBoard()
{
  background(95, 200, 70);
  fill(0);
  ellipse(holeX, holeY, holeRadius*2, holeRadius*2);
  fill(255);
  ellipse(ballX, ballY, ballRadius*2, ballRadius*2);
}

// Do the actual physical simulation.  As usual, this 
// depends on a few global values: the current position
// and velocity of the ball, and the friction with the
// putting green.  Don't forget to process collisions --
// that means you'll also be referring to the width
// and height of the sketch, and the size of the ball
// and hole.
void step()
{
  // TODO: Fill in this physical simulation.  Note that
  // when you first run this code, the program will get
  // stuck, because there's nothing to tell the sketch
  // to get out of SIMULATE mode.
  
  // IF YOU'RE STUCK, try adding features to the
  // simulation as follows:
  
  // 1. Implement the first law: get the ball rolling
  // with constant velocity in a fixed direction.  It'll
  // just roll right off the screen.
  
  // 2. Implement the third law: get the ball colliding
  // off the four sides of the sketch window.
  
  // 3. Implement friction/damping/drag/etc.  Choose
  // a method for gradually decreasing the velocity
  // until the ball comes to a stop.  Note that unlike
  // the sample sketches from the lectures, you'll want
  // to explicitly detect when the ball has slowed nearly
  // to a stop, and force it to stop completely.  If
  // the ball comes to a stop, move back to the READY
  // state (allowing the user to line up a new putt).
  
  // Check if the ball is inside the hole.  If so, 
  // it indicates successful completion of this level.
  if( dist( ballX, ballY, holeX, holeY ) < holeRadius ) {
    state = STATE_CELEBRATE;
  }
}

void draw()
{
  if( state == STATE_READY ) {
    drawBoard();
  } else if( state == STATE_CELEBRATE ) {
    // The way this state is written, we immediately reset and
    // start a new level.  But we left "CELEBRATE" mode here
    // in case you would like to implement some kind of 
    // animation to indicate that the current level was
    // solved.
    resetGame();
    state = STATE_READY;
  } else if( state == STATE_WINDUP ) {
    drawBoard();
    stroke(200);
    line(ballX, ballY, mouseX, mouseY);
    noStroke();
  } else if( state == STATE_SIMULATE ) {
    step();
    drawBoard();
  }
}

void mousePressed()
{
  // There's only one mouse press we care about -- we're at
  // the start of a stroke, and the user clicks on the ball
  // to begin a stroke.
  
  if( state == STATE_READY ) {
    // Hit test against ball.
    if( dist( mouseX, mouseY, ballX, ballY ) < ballRadius ) {
      state = STATE_WINDUP;
    }
  }
}

void mouseReleased()
{
  // If you're in the middle of winding up a stroke,
  // Turn it into an actual stroke (putt) by imparting
  // some initial velocity to the ball.
  
  if( state == STATE_WINDUP ) {
    float len = dist( mouseX, mouseY, ballX, ballY );
    
    // TODO: set the initial velocity of the ball.  Our
    // initial estimate of the putt power is the length
    // of the line from the mouse to the ball, measured
    // in pixels.  First, "clamp" this value so that 
    // it's no smaller than minPower and no larger than
    // maxPower.  Then convert it into a desired speed
    // using speedNorm.  Then set the horizontal and
    // vertical components of the velocity so that they
    // correspond to an arrow in the direction from the
    // mouse to the ball, of the total speed computed
    // above.
    
    state = STATE_SIMULATE;
  }
}
